/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Eina.h>
#include "$(appName).h"
#include "data.h"

static bool _create_record(const char *uri, calendar_record_h *rec_h);
static bool _add_child_record(calendar_record_h rec_h, int field_id, calendar_record_h child_h);
static bool _get_child_record_first(calendar_record_h rec_h, int field_id, calendar_record_h *child_h);
static bool _get_child_record_p(calendar_record_h rec_h, int field_id, int index, calendar_record_h *child_h);
static bool _find_child_record_ext(calendar_record_h rec_h, const char *child_key, calendar_record_h *child_h);
static bool _get_child_record_count(calendar_record_h rec_h, int field_id, unsigned int *count);
static bool _set_str(calendar_record_h rec_h, int field_id, const char *value);
static bool _get_str(calendar_record_h rec_h, int field_id, char **value);
static bool _set_time(calendar_record_h rec_h, int field_id, struct tm *value, bool is_allday);
static bool _get_time(calendar_record_h rec_h, int field_id, struct tm *value, bool *is_all_day);
static bool _set_int(calendar_record_h rec_h, int field_id, int value);
static bool _get_int(calendar_record_h rec_h, int field_id, int *value);
static bool _set_double(calendar_record_h rec_h, int field_id, double value);
static bool _get_record_from_list(calendar_list_h list_h, calendar_record_h *rec_h);
static bool _assign_string_value(char **dest, char *value);

/**
 * @brief Function connects to the calendar database.
 * @return This function returns 'true' if successfully connected to the calendar database,
 * otherwise 'false' is returned.
 */
bool data_connect_calendar(void)
{
	int ret = calendar_connect();
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_connect() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Function disconnects from the calendar database.
 * @return This function returns 'true' if successfully disconnected from the calendar database,
 * otherwise 'false' is returned.
 */
bool data_disconnect_calendar(void)
{
	int ret = calendar_disconnect();
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_disconnect() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Function creates event record for calendar database and sets basic data to this record.
 * @param[out] rec_h The newly created record's handle.
 * @return This function returns 'true' if the record was successfully created,
 * otherwise 'false' is returned.
 */
bool data_create_record(calendar_record_h *rec_h)
{
	int ret;

	if (!rec_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_create_record() failed. Invalid input argument.");
		return false;
	}

	if (!_create_record("tizen.calendar_view.event", rec_h))
		return false;

	ret = calendar_record_set_str(*rec_h, _calendar_event.organizer_name, "0");
	if (ret != CALENDAR_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_set_str() failed. Err = %d.", ret);
		calendar_record_destroy(*rec_h, EINA_TRUE);
		return false;
	}

	ret = calendar_record_set_int(*rec_h, _calendar_event.calendar_book_id, DEFAULT_EVENT_CALENDAR_BOOK_ID);
	if (ret != CALENDAR_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_set_int() failed. Err = %d.", ret);
		calendar_record_destroy(*rec_h, EINA_TRUE);
		return false;
	}

	return true;
}

/**
 * @brief Function destroys record.
 * @param[in] rec_h The record's handle to be destroyed.
 * @return This function returns 'true' if the record was successfully destroyed,
 * otherwise 'false' is returned.
 */
bool data_destroy_record(calendar_record_h rec_h)
{
	int ret = calendar_record_destroy(rec_h, EINA_TRUE);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_destroy() failed. Err = %d.", ret);

	return (ret != CALENDAR_ERROR_NONE);
}

/**
 * @brief Function inserts record to the calendar database.
 * @param[in] rec_h The record's handle to be inserted to the database.
 * @return This function returns 'true' if the record was successfully inserted to the database,
 * otherwise 'false' is returned.
 */
bool data_insert_record(calendar_record_h rec_h)
{
	int rec_index = 0;
	int ret = calendar_db_insert_record(rec_h, &rec_index);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_db_insert_record() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Function updates the record in the calendar database.
 * @param[in] rec_h The record's handle to be updated in the database.
 * @return This function returns 'true' if the record was successfully updated in the database,
 * otherwise 'false' is returned.
 */
bool data_update_record(calendar_record_h rec_h)
{
	int ret = calendar_db_update_record(rec_h);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_db_update_record() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Function deletes record from the calendar database.
 * @param[in] rec_id The identifier of the record to be deleted from the database.
 * @return This function returns 'true' if the record was successfully deleted from the database,
 * otherwise 'false' is returned.
 */
bool data_delete_record(int rec_id)
{
	int ret;

	ret = calendar_db_delete_record(_calendar_event._uri, rec_id);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_db_delete_record(%d) failed. Err = %d.", rec_id, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Function gets record by its id from the calendar database.
 * @param[in] rec_id The identifier of the record to be obtained from the database.
 * @param[out] rec_h The obtained record handle.
 * @return This function returns 'true' if the record was successfully obtained from the database,
 * otherwise 'false' is returned.
 */
bool data_get_record(int rec_id, calendar_record_h *rec_h)
{
	int ret;

	ret = calendar_db_get_record(_calendar_event._uri, rec_id, rec_h);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_db_get_record(%d) failed. Err = %d.", rec_id, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief This function adds a set of data to the record of calendar database.
 * @param[in] title The event's title.
 * @param[in] t_start The pointer to the event's start time structure.
 * @param[in] t_end The pointer to the event's end time structure.
 * @param[in] all_day The flag indicating that the event lasts all the day.
 * @param[in] location The event's location.
 * @param[in] description The event's description.
 * @param[in] priority The event's priority.
 * @param[in] sensitivity The event's sensitivity.
 * @param[in] status The event's status.
 * @param[in] recurr_freq The frequency of recurring alarm.
 * @param[in] recurr_range The range of recurring alarm.
 * @param[in] recurr_count The number of recurrences for the alarm.
 * @param[in] recurr_until Alarm recurrence due time.
 * @param[in] reminder Reminder timeout expressed in minutes.
 * @return This function returns 'true' if the data was successfully set to the record,
 * otherwise 'false' is returned.
 */
bool data_set_record_data(calendar_record_h rec_h, const char *title, struct tm *t_start, struct tm *t_end, bool all_day,
							const char *location, const char *description, int priority, int sensitivity, int status,
							int recurr_freq, int recurr_range, int recurr_count, struct tm *recurr_until, int reminder)
{
	char buff[128] = {0,};
	calendar_record_h child_h = NULL;
	bool rec_exists;
	int tick_unit;
	time_t t = time(NULL);
	struct tm *tm_local_time = localtime(&t);

	_set_str(rec_h, _calendar_event.summary, title);
	_set_time(rec_h, _calendar_event.start_time, t_start, all_day);
	_set_time(rec_h, _calendar_event.end_time, t_end, all_day);

	strftime(buff, sizeof(buff), "%Z, %z", tm_local_time);
	_set_str(rec_h, _calendar_event.start_tzid, buff);
	_set_str(rec_h, _calendar_event.end_tzid, buff);

	rec_exists = _find_child_record_ext(rec_h, "city", &child_h);
	if (!rec_exists)
		if (_create_record(_calendar_extended_property._uri, &child_h))
			rec_exists = _add_child_record(rec_h, _calendar_event.extended, child_h);

	if (rec_exists) {
		_set_str(child_h, _calendar_extended_property.key, "city");
		_set_str(child_h, _calendar_extended_property.value, "IDS_WCL_BODY_CITYNAME_SEOUL");
	}

	_set_str(rec_h, _calendar_event.location, location);
	_set_double(rec_h, _calendar_event.latitude, CALENDAR_RECORD_NO_COORDINATE);
	_set_double(rec_h, _calendar_event.longitude, CALENDAR_RECORD_NO_COORDINATE);
	_set_str(rec_h, _calendar_event.description, description);
	_set_int(rec_h, _calendar_event.priority, priority);

	_set_int(rec_h, _calendar_event.sensitivity, sensitivity);
	_set_int(rec_h, _calendar_event.event_status, status);

	snprintf(buff, sizeof(buff), "%d", recurr_freq);
	_set_str(rec_h, _calendar_event.recurrence_id, buff);
	_set_int(rec_h, _calendar_event.freq, recurr_freq);
	_set_int(rec_h, _calendar_event.interval, 1);

	if (recurr_freq != CALENDAR_RECURRENCE_NONE) {
		_set_int(rec_h, _calendar_event.range_type, recurr_range);
		_set_int(rec_h, _calendar_event.count, recurr_count);
		_set_time(rec_h, _calendar_event.until_time, recurr_until, false);
	}

	rec_exists = _get_child_record_first(rec_h, _calendar_event.calendar_alarm, &child_h);
	if (!rec_exists)
		if (_create_record(_calendar_alarm._uri, &child_h))
			rec_exists = _add_child_record(rec_h, _calendar_event.calendar_alarm, child_h);

	if (rec_exists) {
		if (reminder == -1)
			tick_unit = CALENDAR_ALARM_NONE;
		else
			tick_unit = CALENDAR_ALARM_TIME_UNIT_MINUTE;

		_set_int(child_h, _calendar_alarm.tick_unit, tick_unit);
		_set_int(child_h, _calendar_alarm.tick, reminder);
	}

	return true;
}

/**
 * @brief This function gets a set of data from the record of calendar database.
 * @param[in] rec_h The record handle used for data extraction.
 * @param[out] rec_info Pointer to the allocated structure containing all the acquired data.
 * This pointer must be released with data_destroy_record_data() after it is used.
 * @return This function returns 'true' if the data was successfully acquired,
 * otherwise 'false' is returned.
 */
bool data_get_record_data(calendar_record_h rec_h, rec_info_s **rec_info)
{
	char *buff = NULL;
	calendar_record_h child_h;

	*rec_info = (rec_info_s *)calloc(1, sizeof(rec_info_s));
	if (!*rec_info) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_get_record_data() failed. Memory allocation error.");
		return false;
	}

	_get_int(rec_h, _calendar_event.id, &(*rec_info)->id);

	if (_get_str(rec_h, _calendar_event.summary, &buff))
		_assign_string_value(&(*rec_info)->title, buff);

	_get_time(rec_h, _calendar_event.start_time, &(*rec_info)->t_start, &(*rec_info)->all_day);
	_get_time(rec_h, _calendar_event.end_time, &(*rec_info)->t_end, NULL);

	if (_get_str(rec_h, _calendar_event.start_tzid, &buff))
		_assign_string_value(&(*rec_info)->time_zone, buff);

	if (_get_str(rec_h, _calendar_event.location, &buff))
		_assign_string_value(&(*rec_info)->location, buff);

	if (_get_str(rec_h, _calendar_event.description, &buff))
		_assign_string_value(&(*rec_info)->description, buff);

	_get_int(rec_h, _calendar_event.freq, &(*rec_info)->recurr_freq);
	_get_int(rec_h, _calendar_event.range_type, &(*rec_info)->recurr_range);
	_get_int(rec_h, _calendar_event.count, &(*rec_info)->recurr_count);
	_get_time(rec_h, _calendar_event.until_time, &(*rec_info)->recurr_until, NULL);
	_get_int(rec_h, _calendar_event.priority, &(*rec_info)->priority);
	_get_int(rec_h, _calendar_event.sensitivity, &(*rec_info)->sensitivity);
	_get_int(rec_h, _calendar_event.event_status , &(*rec_info)->status);

	if (_get_child_record_first(rec_h, _calendar_event.calendar_alarm, &child_h))
		_get_int(child_h, _calendar_alarm.tick, &(*rec_info)->reminder);

	return true;
}

/**
 * @brief Function destroys the record data acquired with data_get_record_data() function.
 * @param[in] rec_info The pointer to the structure to be destroyed.
 */
void data_destroy_record_data(rec_info_s *rec_info)
{
	if (!rec_info)
		return;

	free(rec_info->title);
	free(rec_info->time_zone);
	free(rec_info);
}

/**
 * @brief Function gets all the records stored in calendar database under provided uri.
 * @param[in] uri The URI of the database view.
 * @param[out] list_h The all records list handle.
 * @param[out] count The number of records stored in a list.
 * @return This function returns 'true' if the records list was obtained successfully,
 * otherwise 'false' is returned.
 */
bool data_get_all_records(const char *uri, calendar_list_h *list_h, int *count)
{
	int ret;

	if (!list_h || !count) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_get_all_records() failed. Invalid input argument.");
		return false;
	}

	*list_h = NULL;
	*count = 0;

	ret = calendar_db_get_all_records(uri, 0, 0, list_h);
	if (ret != CALENDAR_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_db_get_all_records() failed. Err = %d.", ret);
		return false;
	}

	ret = calendar_list_get_count(*list_h, count);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_list_get_count() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Function gets the first record from the list of records.
 * @param[in] list_h The records list handle.
 * @param[out] rec_h The record handle obtained from the list.
 * @return This function returns 'true' if the record was obtained successfully,
 * otherwise 'false' is returned.
 */
bool data_get_first_list_item(calendar_list_h list_h, calendar_record_h *rec_h)
{
	int ret;

	*rec_h = NULL;

	ret = calendar_list_first(list_h);
	if (ret != CALENDAR_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_list_first() failed. Err = %d.", ret);
		return false;
	}

	return _get_record_from_list(list_h, rec_h);
}

/**
 * @brief Function gets the next record from the list of records.
 * @param[in] list_h The records list handle.
 * @param[out] rec_h The record handle obtained from the list.
 * @return This function returns 'true' if the record was obtained successfully,
 * otherwise 'false' is returned.
 */
bool data_get_next_list_item(calendar_list_h list_h, calendar_record_h *rec_h)
{
	int ret;

	*rec_h = NULL;

	ret = calendar_list_next(list_h);
	if (ret != CALENDAR_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_list_next() failed. Err = %d.", ret);
		return false;
	}

	return _get_record_from_list(list_h, rec_h);
}

/**
 * @brief Function creates new record for calendar database.
 * @param[out] rec_h The newly created record's handle.
 * @return This function returns 'true' if the record was successfully created,
 * otherwise 'false' is returned.
 */
static bool _create_record(const char *uri, calendar_record_h *rec_h)
{
	int ret = calendar_record_create(uri, rec_h);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_create(%s) failed. Err = %d.", uri, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which adds a child to the given record at provided field.
 * @param[in] rec_h The parent record for a child.
 * @param[in] field_id The field identifier for child record.
 * @param[out] child_h The handle of child record to be added.
 * @return This function returns 'true' if the child record was successfully added,
 * otherwise 'false' is returned.
 */
static bool _add_child_record(calendar_record_h rec_h, int field_id, calendar_record_h child_h)
{
	int ret = calendar_record_add_child_record(rec_h, field_id, child_h);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_add_child_record() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which gets a child form given record stored under provided field identifier.
 * @param[in] rec_h The parent record.
 * @param[in] field_id The field identifier for child record.
 * @param[out] child_h The handle of child record acquired.
 * @return This function returns 'true' if the child record was successfully acquired,
 * otherwise 'false' is returned.
 */
static bool _get_child_record_first(calendar_record_h rec_h, int field_id, calendar_record_h *child_h)
{
	unsigned int count = 0;

	if (!child_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_get_child_record() failed. Invalid input argument.");
		return false;
	}

	if (!_get_child_record_count(rec_h, field_id, &count))
		return false;

	return (count > 0 && _get_child_record_p(rec_h, field_id, 0, child_h));
}

/**
 * @brief Internal function which gets the pointer to the child form given record
 * stored under provided field identifier at specified index.
 * @param[in] rec_h The parent record.
 * @param[in] field_id The field identifier for child record.
 * @param[in] index The index of record under the parent in provided field_id.
 * @param[out] child_h The handle of child record acquired.
 * @return This function returns 'true' if the child record was successfully acquired,
 * otherwise 'false' is returned.
 */
static bool _get_child_record_p(calendar_record_h rec_h, int field_id, int index, calendar_record_h *child_h)
{
	int ret = calendar_record_get_child_record_at_p(rec_h, field_id, index, child_h);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_child_record_at_p() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which gets a child form given record stored under extended property.
 * @param[in] rec_h The parent record of searched child.
 * @param[in] child_key The key name of child record.
 * @param[out] child_h The handle of child record found.
 * @return This function returns 'true' if the child record was successfully found,
 * otherwise 'false' is returned.
 */
static bool _find_child_record_ext(calendar_record_h rec_h, const char *child_key, calendar_record_h *child_h)
{
	int i;
	int ret;
	char *key;
	unsigned int child_count = 0;

	if (!child_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_find_child_record_ext() failed. Invalid input argument.");
		return false;
	}

	*child_h = NULL;

	if (!_get_child_record_count(rec_h, _calendar_event.extended, &child_count))
		return false;

	for (i = 0; i < child_count; i++) {
		if (!_get_child_record_p(rec_h, _calendar_event.extended, i, child_h))
			continue;

		key = NULL;
		ret = calendar_record_get_str_p(*child_h, _calendar_extended_property.key, &key);
		if (ret != CALENDAR_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_str_p() failed. Err = %d.", ret);
			continue;
		}

		if (key && !strcmp(key, child_key))
			return true;
	}

	*child_h = NULL;
	return false;
}

/**
 * @brief Internal function which gets the number of children form given record stored under
 * property of provided identifier.
 * @param[in] rec_h The parent record.
 * @param[in] field_id The identifier of the field for child.
 * @param[out] count The number of children under parent record.
 * @return This function returns 'true' if the number of children was successfully obtained,
 * otherwise 'false' is returned.
 */
static bool _get_child_record_count(calendar_record_h rec_h, int field_id, unsigned int *count)
{
	int ret = calendar_record_get_child_record_count(rec_h, field_id, count);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_child_record_count() failed. Err = %d.", ret);

	return (ret != CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which sets the string value to the provided record of calendar database.
 * @param[in] rec_h The record's handle to be altered with new data.
 * @param[in] field_id The identifier of the field where the new data is to be set.
 * @param[in] value The value to be set to the designated field of provided database record.
 * @return This function returns 'true' if the data was successfully set to the record,
 * otherwise 'false' is returned.
 */
static bool _set_str(calendar_record_h rec_h, int field_id, const char *value)
{
	int ret = calendar_record_set_str(rec_h, field_id, value);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_set_str(%s) failed. Err = %d.", value, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which gets the string value from the provided record of calendar database.
 * @param[in] rec_h The record's handle.
 * @param[in] field_id The identifier of the field from which the data is to be obtained.
 * @param[out] value The obtained value.
 * @return This function returns 'true' if the data was successfully obtained from the record,
 * otherwise 'false' is returned.
 */
static bool _get_str(calendar_record_h rec_h, int field_id, char **value)
{
	int ret = calendar_record_get_str(rec_h, field_id, value);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_str() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which sets the time value to the provided record of calendar database.
 * @param[in] rec_h The record's handle to be altered with new data.
 * @param[in] field_id The identifier of the field where the new data is to be set.
 * @param[in] value The value to be set to the designated field of provided database record.
 * @return This function returns 'true' if the data was successfully set to the record,
 * otherwise 'false' is returned.
 */
static bool _set_time(calendar_record_h rec_h, int field_id, struct tm *value, bool is_allday)
{
	int ret;
	calendar_time_s t = {0,};

	if (!value) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_set_time() failed. Invalid input argument.");
		return false;
	}

	ret = calendar_record_get_caltime(rec_h, field_id, &t);
	if (ret != CALENDAR_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_caltime() failed. Err = %d.", ret);
		return false;
	}

	if (is_allday) {
		t.type = CALENDAR_TIME_LOCALTIME;
		t.time.date.year = value->tm_year + 1900;
		t.time.date.month = value->tm_mon + 1;
		t.time.date.mday = value->tm_mday;
	} else {
		value->tm_sec = 0;

		t.type = CALENDAR_TIME_UTIME;
		t.time.utime = mktime(value);
	}

	ret = calendar_record_set_caltime(rec_h, field_id, t);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_set_caltime() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which gets the time value from the provided record of calendar database.
 * @param[in] rec_h The record's handle.
 * @param[in] field_id The identifier of the field from which the data is to be obtained.
 * @param[out] value The obtained value.
 * @param[out] all_day The flag indicates whether an event is an all day event.
 * @return This function returns 'true' if the data was successfully obtained from the record,
 * otherwise 'false' is returned.
 */
static bool _get_time(calendar_record_h rec_h, int field_id, struct tm *value, bool *is_all_day)
{
	int ret;
	calendar_time_s t = {0,};

	if (is_all_day)
		*is_all_day = false;

	ret = calendar_record_get_caltime(rec_h, field_id, &t);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_caltime() failed. Err = %d.", ret);

	if (t.type == CALENDAR_TIME_UTIME) {
		*value = *localtime((const time_t *)&t.time.utime);
	} else if (t.type == CALENDAR_TIME_LOCALTIME) {
		value->tm_year = t.time.date.year - 1900;
		value->tm_mon = t.time.date.month - 1;
		value->tm_mday = t.time.date.mday;

		if (is_all_day)
			*is_all_day = (field_id == _calendar_event.start_time);
	}

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which sets the integer value to the provided record of calendar database.
 * @param[in] rec_h The record's handle to be altered with new data.
 * @param[in] field_id The identifier of the field where the new data is to be set.
 * @param[in] value The value to be set to the designated field of provided database record.
 * @return This function returns 'true' if the data was successfully set to the record,
 * otherwise 'false' is returned.
 */
static bool _set_int(calendar_record_h rec_h, int field_id, int value)
{
	int ret = calendar_record_set_int(rec_h, field_id, value);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_set_int(%d) failed. Err = %d.", field_id, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which gets the integer value from the provided record of calendar database.
 * @param[in] rec_h The record's handle.
 * @param[in] field_id The identifier of the field from which the data is to be obtained.
 * @param[out] value The obtained value.
 * @return This function returns 'true' if the data was successfully obtained from the record,
 * otherwise 'false' is returned.
 */
static bool _get_int(calendar_record_h rec_h, int field_id, int *value)
{
	int ret = calendar_record_get_int(rec_h, field_id, value);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_get_int(%d) failed. Err = %d.", field_id, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which sets the integer value to the provided record of calendar database.
 * @param[in] rec_h The record's handle to be altered with new data.
 * @param[in] field_id The identifier of the field where the new data is to be set.
 * @param[in] value The value to be set to the designated field of provided database record.
 * @return This function returns 'true' if the data was successfully set to the record,
 * otherwise 'false' is returned.
 */
static bool _set_double(calendar_record_h rec_h, int field_id, double value)
{
	int ret = calendar_record_set_double(rec_h, field_id, value);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_record_set_double(%d) failed. Err = %d.", field_id, ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which gets the current record from the list.
 * @param[in] list_h The list's handle.
 * @param[in] rec_h The pointer to the obtained record's handle.
 * @return This function returns 'true' if the record was successfully obtained,
 * otherwise 'false' is returned.
 */
static bool _get_record_from_list(calendar_list_h list_h, calendar_record_h *rec_h)
{
	int ret = calendar_list_get_current_record_p(list_h, rec_h);
	if (ret != CALENDAR_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "calendar_list_get_current_record_p() failed. Err = %d.", ret);

	return (ret == CALENDAR_ERROR_NONE);
}

/**
 * @brief Internal function which duplicates the string argument and assigns it
 * to the destination argument. This function validates the string being assigned
 * and frees it after the operation succeeds.
 * @param[out] dest The pointer to the destination string.
 * @param[in] value The pointer to the value to be assigned to the destination.
 * @return This function returns 'true' if the operation is successful,
 * otherwise 'false' is returned.
 */
static bool _assign_string_value(char **dest, char *value)
{
	if (!value)
		return false;

	*dest = strdup(value);
	free(value);

	return true;
}
