/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <calendar.h>
#include <efl_extension.h>
#include "$(appName).h"
#include "view.h"
#include "view_defines.h"
#include "view/view_event_details.h"
#include "view/view_event_edit.h"

typedef struct _item_content {
	char *label_caption;
	char *text;
} item_content_s;

static struct view_info {
	Evas_Object *navi;
	Evas_Object *layout;
	Evas_Object *genlist;
	Evas_Object *popup;
	Elm_Object_Item *navi_item;
	rec_info_s *rec_info;
	event_deleted_cb deleted_cb;
} s_info = {
	.navi = NULL,
	.layout = NULL,
	.genlist = NULL,
	.popup = NULL,
	.navi_item = NULL,
	.rec_info = NULL,
	.deleted_cb = NULL,
};

static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info);
static Eina_Bool _navi_pop_cb(void *data, Elm_Object_Item *it);
static void _popup_ok_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _popup_cancel_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _edited_event_saved_cb(void);
static void _edit_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _delete_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _append_items(void);
static void _remove_items(void);
static rec_info_s *_get_rec_info(int rec_id);

/**
 * @brief Adds event details view.
 * @param[in] navi Main naviframe object.
 * @param[in] rec_info Pointer to the structure containing all the event's information.
 * @return This function returns a layout object if is was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_event_details_create(Evas_Object *navi, rec_info_s *rec_info, event_deleted_cb func)
{
	Evas_Object *box;
	Evas_Object *button;

	s_info.navi = navi;
	s_info.rec_info = rec_info;
	s_info.deleted_cb = func;

	s_info.layout = view_create_layout(s_info.navi, EDJ_EVENT_DETAILS, GRP_EVENT_DETAILS);
	if (!s_info.layout)
		return NULL;

	elm_layout_theme_set(s_info.layout, "layout", "application", "default");

	s_info.genlist = view_create_genlist(s_info.layout, PART_EVENT_DETAILS_LIST);
	if (!s_info.genlist)
		return NULL;

	_append_items();

	s_info.navi_item = elm_naviframe_item_push(s_info.navi, "Details", NULL, NULL, s_info.layout, NULL);
	elm_naviframe_item_pop_cb_set(s_info.navi_item, _navi_pop_cb, NULL);

	box = view_create_box(s_info.navi);
	if (!box)
		return NULL;

	button = view_create_button(s_info.navi, "Edit", NULL, _edit_button_clicked_cb, NULL);
	if (!button)
		return NULL;

	elm_box_pack_end(box, button);

	button = view_create_button(s_info.navi, "Delete", NULL, _delete_button_clicked_cb, NULL);
	if (!button)
		return NULL;

	elm_box_pack_end(box, button);
	elm_object_item_part_content_set(s_info.navi_item, "toolbar", box);

	return s_info.layout;
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal callback function invoked on top item removal from the naviframe.
 * This function is responsible for active popup deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] it The naviframe item being popped.
 * @return This function returns 'EINA_TRUE' to allow naviframe item popping
 * or 'EINA_FALSE' to cancel this operation.
 */
static Eina_Bool _navi_pop_cb(void *data, Elm_Object_Item *it)
{
	if (s_info.popup) {
		evas_object_del(s_info.popup);
		s_info.popup = NULL;
	}

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on 'Ok' button click from popup window.
 * This function is responsible for currently viewed record deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_ok_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	bool deleted = false;

	if (data_connect_calendar()) {
		deleted = data_delete_record(s_info.rec_info->id);
		data_disconnect_calendar();
	}

	_popup_cancel_click_cb(data, obj, event_info);

	if (deleted) {
		elm_naviframe_item_pop(s_info.navi);

		if (!s_info.deleted_cb) {
			dlog_print(DLOG_ERROR, LOG_TAG, "event_deleted_cb callback function is not assigned.");
			return;
		}

		s_info.deleted_cb();
	}
}

/**
 * @brief Internal callback function invoked on 'Cancel' button click from popup window.
 * This function is responsible for active popup deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_cancel_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (!s_info.popup)
		return;

	evas_object_del(s_info.popup);
	s_info.popup = NULL;
}

/**
 * @brief Internal callback function invoked by the new event edition view module
 * on successful event update.
 */
static void _edited_event_saved_cb(void)
{
	_remove_items();
	s_info.rec_info = _get_rec_info(s_info.rec_info->id);
	_append_items();

	elm_naviframe_item_pop(s_info.navi_item);
}

/**
 * @brief Internal callback function invoked on 'Edit' button click.
 * This function is responsible for event edition view display.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _edit_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *layout = view_event_edit_create("Edit event", s_info.navi, s_info.rec_info, _edited_event_saved_cb);
	eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);
}

/**
 * @brief Internal callback function invoked on 'Delete' button click.
 * This function is responsible for event removal popup window display.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_info.popup = view_create_popup("Warning", "Do you want to delete this record?", _popup_ok_click_cb, _popup_cancel_click_cb);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * It is responsible for text label display.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_get_text_cb(void *data, Evas_Object *obj, const char *part)
{
	item_content_s *item = (item_content_s *)data;

	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_get_text_cb() failed. Invalid input argument.");
		return NULL;
	}

	return strdup(item->label_caption);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * It is responsible for item layout creation.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The item's layout is returned.
 */
static Evas_Object *_get_item_content_cb(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *label;
	item_content_s *item = (item_content_s *)data;

	if (strcmp(part, "elm.swallow.end"))
		return NULL;

	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_get_item_content_cb() failed. Invalid input argument.");
		return NULL;
	}

	label = view_create_label(obj, item->text, NULL);
	if (!label)
		return NULL;

	return label;
}

/**
 * @brief Internal callback function invoked on genlist item removal.
 * This function is assigned to the func.del handler of genlist item's class structure.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 */
static void _item_del_cb(void *data, Evas_Object *obj)
{
	item_content_s *item_content = (item_content_s *)data;

	if (!item_content) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_item_del_cb() failed. Invalid input argument.");
		return;
	}

	free(item_content->label_caption);
	free(item_content->text);
	free(item_content);
}

/**
 * @brief Internal function which appends new item to the genlist.
 * @param[in] content The text content to be passed to the items class callback function.
  * @return The function returns the item object if it was successfully appended to the genlist,
 * otherwise NULL is returned.
 */
static Elm_Object_Item *_append_item(item_content_s *content)
{
	Elm_Genlist_Item_Class *itc;
	Elm_Object_Item *item;

	itc = create_genlist_item_class("default", _get_text_cb, _get_item_content_cb);
	if (!itc)
		return NULL;

	itc->func.del = _item_del_cb;

	item = elm_genlist_item_append(s_info.genlist, itc, (void *)content, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_append() failed.");
		return NULL;
	}

	elm_genlist_item_class_free(itc);

	return item;
}

/**
 * @brief Internal function which creates the item content structure used by the
 * genlist's item class callback functions to create and display an item.
 * @param[in] label_content The static text labeling a displayed item.
 * @param[in] text The text to be displayed aside the label's caption.
 * @return The item's content structure is returned if function succeeds,
 * otherwise NULL is returned.
 */
static item_content_s *_create_item_content(const char *label_caption, char *text)
{
	if (!label_caption) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_item_content() failed. Invalid input argument.");
		return NULL;
	}

	item_content_s *item = (item_content_s *)calloc(1, sizeof(item_content_s));
	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_item_content() failed. Memory allocation error.");
		return NULL;
	}

	item->label_caption = strdup(label_caption);

	if (text)
		item->text = strdup(text);

	return item;
}

/**
 * @brief Internal function which appends the set of items to the genlist.
 * @return This function returns 'true' if all the items were successfully appended to the genlist,
 * otherwise 'false' is returned.
 */
static void _append_items(void)
{
	item_content_s *item;
	char buff[32] = {0,};

	_append_item(_create_item_content("Title", s_info.rec_info->title));

	strftime(buff, sizeof(buff), "%d %b %Y, %R", &s_info.rec_info->t_start);
	_append_item(_create_item_content("Start", buff));

	strftime(buff, sizeof(buff), "%d %b %Y, %R", &s_info.rec_info->t_end);
	_append_item(_create_item_content("End", buff));
	_append_item(_create_item_content("Time zone", s_info.rec_info->time_zone));
	_append_item(_create_item_content("Location", s_info.rec_info->location));
	_append_item(_create_item_content("Description", s_info.rec_info->description));
	_append_item(_create_item_content("All day", s_info.rec_info->all_day ? "Yes" : "No"));

	item = _create_item_content("Recurrence", s_info.rec_info->recurr_freq == CALENDAR_RECURRENCE_NONE ? "None" :
												s_info.rec_info->recurr_freq == CALENDAR_RECURRENCE_DAILY ? "Daily" :
												s_info.rec_info->recurr_freq == CALENDAR_RECURRENCE_WEEKLY ? "Weekly" :
												s_info.rec_info->recurr_freq == CALENDAR_RECURRENCE_MONTHLY ? "Monthly" :
												s_info.rec_info->recurr_freq == CALENDAR_RECURRENCE_YEARLY ? "Yearly" :
												"Undefined");
	_append_item(item);

	if (s_info.rec_info->reminder > 0) {
		snprintf(buff, sizeof(buff), "%d [min]", s_info.rec_info->reminder);
		item = _create_item_content("Reminder", buff);
	} else {
		item = _create_item_content("Reminder", "None");
	}
	_append_item(item);

	item = _create_item_content("Priority", s_info.rec_info->priority == CALENDAR_EVENT_PRIORITY_NONE ? "None" :
												s_info.rec_info->priority == CALENDAR_EVENT_PRIORITY_LOW ? "Low" :
												s_info.rec_info->priority == CALENDAR_EVENT_PRIORITY_NORMAL ? "Normal" :
												s_info.rec_info->priority == CALENDAR_EVENT_PRIORITY_HIGH ? "High" :
												"Undefined");
	_append_item(item);

	item = _create_item_content("Sensitivity", s_info.rec_info->sensitivity == CALENDAR_SENSITIVITY_PUBLIC ? "Public" :
												s_info.rec_info->sensitivity == CALENDAR_SENSITIVITY_PRIVATE ? "Private" :
												s_info.rec_info->sensitivity == CALENDAR_SENSITIVITY_CONFIDENTIAL ? "Confidential" :
												"Undefined");
	_append_item(item);

	item = _create_item_content("Status", s_info.rec_info->status == CALENDAR_EVENT_STATUS_NONE ? "None" :
												s_info.rec_info->status == CALENDAR_EVENT_STATUS_TENTATIVE ? "Tentative" :
												s_info.rec_info->status == CALENDAR_EVENT_STATUS_CONFIRMED ? "Confirmed" :
												s_info.rec_info->status == CALENDAR_EVENT_STATUS_CANCELLED ? "Canceled" :
												"Undefined");
	_append_item(item);
}

/**
 * @brief Internal function which removes all items from the genlist.
 */
static void _remove_items(void)
{
	elm_genlist_clear(s_info.genlist);
}

/**
 * @brief Internal function which returns calendar event with required id.
 * @param[in] rec_id Id if required calendar event..
 * @return This function returns structure with calendar event data if rec_id exists,
 * otherwise 'NULL' is returned.
 */
static rec_info_s *_get_rec_info(int rec_id)
{
	int count = 0;
	calendar_list_h list_h;
	calendar_record_h rec_h;
	rec_info_s *rec_info;

	if (!data_connect_calendar()) {
		return NULL;
	}

	bool listObtained = data_get_all_records(_calendar_event._uri, &list_h, &count);
	bool firstRecordObtained = data_get_first_list_item(list_h, &rec_h);
	if (!listObtained || !firstRecordObtained || count == 0 || !rec_h) {
		data_disconnect_calendar();
		return NULL;
	}

	do {
		rec_info = NULL;

		if (!data_get_record_data(rec_h, &rec_info))
			continue;

		if (rec_info->id == rec_id) {
			break;
		}
	} while(data_get_next_list_item(list_h, &rec_h));

	calendar_list_destroy(list_h, true);
	data_disconnect_calendar();

	return rec_info;
}
