/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <time.h>
#include <calendar.h>
#include "$(appName).h"
#include "view.h"
#include "view_defines.h"
#include "view/view_event_edit.h"
#include "view/view_recurrence.h"

#define BUTTON_WIDTH 320
#define BUTTON_HEIGHT 80
#define BUTTON_DATE_WIDTH 240
#define BUTTON_TIME_WIDTH 160

typedef struct _date_time {
	struct tm *tm;
	const char *format;
} date_time_s;

typedef struct _labels_type {
	int value;
	const char *name;
} label_type_s;

static struct view_info {
	Evas_Object	*win;
	Evas_Object	*naviframe;
	Evas_Object	*layout;
	Evas_Object	*genlist;
	Evas_Object	*entry_title;
	Evas_Object	*entry_description;
	Evas_Object	*entry_location;
	Evas_Object	*reminder_btn;
	Evas_Object	*recurrence_btn;
	Evas_Object	*priority_btn;
	Evas_Object	*sensitivity_btn;
	Evas_Object	*status_btn;
	Evas_Object *popup;
	Elm_Object_Item *genlist_from_item;
	Elm_Object_Item *genlist_to_item;
	Elm_Object_Item *genlist_reminder_item;
	Elm_Object_Item *genlist_location_item;
	Elm_Object_Item *genlist_description_item;
	char *title_text;
	char *location_text;
	char *description_text;
	const char *reminder_type_name;
	const char *recurrence_type_name;
	const char *priority_type_name;
	const char *sensitivity_type_name;
	const char *status_type_name;
	label_type_s reminder_labels[8];
	label_type_s priority_labels[3];
	label_type_s sensitivity_labels[3];
	label_type_s status_labels[4];
	int reminder;
	recurrence_repeat_data recurrence_data;
	calendar_event_priority_e priority;
	calendar_sensitivity_e sensitivity;
	calendar_event_status_e status;
	struct tm stm;
	struct tm etm;
	Eina_Bool is_allday;
	event_saved_cb saved_cb;
} s_info = {
	.win = NULL,
	.naviframe = NULL,
	.layout = NULL,
	.genlist = NULL,
	.entry_title = NULL,
	.entry_description = NULL,
	.entry_location = NULL,
	.reminder_btn = NULL,
	.recurrence_btn = NULL,
	.priority_btn = NULL,
	.sensitivity_btn = NULL,
	.status_btn = NULL,
	.popup = NULL,
	.genlist_from_item = NULL,
	.genlist_to_item = NULL,
	.genlist_reminder_item = NULL,
	.genlist_location_item = NULL,
	.genlist_description_item = NULL,
	.title_text = NULL,
	.location_text = NULL,
	.description_text = NULL,
	.reminder_type_name = "None",
	.recurrence_type_name = "None",
	.priority_type_name = "Low",
	.sensitivity_type_name = "Public",
	.status_type_name = "None",
	.reminder_labels = { {.value = -1, .name = "None"},
							{.value = 0, .name = "At start time"},
							{.value = 5, .name = "5 min before"},
							{.value = 15, .name = "15 min before"},
							{.value = 30, .name = "30 min before"},
							{.value = 60, .name = "1 hour before"},
							{.value = 1440, .name = "1 day before"},
							{.value = 10080, .name = "1 week before"}, },
	.priority_labels = { {.value = CALENDAR_EVENT_PRIORITY_LOW, .name = "Low"},
							{.value = CALENDAR_EVENT_PRIORITY_NORMAL, .name = "Normal"},
							{.value = CALENDAR_EVENT_PRIORITY_HIGH, .name = "High"}, },
	.sensitivity_labels = { {.value = CALENDAR_SENSITIVITY_PUBLIC, .name = "Public"},
							{.value = CALENDAR_SENSITIVITY_PRIVATE, .name = "Private"},
							{.value = CALENDAR_SENSITIVITY_CONFIDENTIAL, .name = "Confidential"}, },
	.status_labels = { {.value = CALENDAR_EVENT_STATUS_NONE, .name = "None"},
						{.value = CALENDAR_EVENT_STATUS_TENTATIVE, .name = "Tentative"},
						{.value = CALENDAR_EVENT_STATUS_CONFIRMED, .name = "Confirmed"},
						{.value = CALENDAR_EVENT_STATUS_CANCELLED, .name = "Canceled"}, },
	.reminder = -1,
	.recurrence_data.id = CALENDAR_RECURRENCE_NONE,
	.recurrence_data.range_type = -1,
	.priority = CALENDAR_EVENT_PRIORITY_LOW,
	.sensitivity = CALENDAR_SENSITIVITY_PUBLIC,
	.status = CALENDAR_EVENT_STATUS_NONE,
	.stm = {0,},
	.etm = {0,},
	.is_allday = EINA_FALSE,
	.saved_cb = NULL,
};

static Eina_Bool _navi_pop_cb(void *data, Elm_Object_Item *it);
static void _entry_changed_cb(void *data, Evas_Object *obj, void *event_info);
static void _save_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _popup_cancel_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _view_initialize(rec_info_s *rec_info);
static const char *_get_recurrence_label(calendar_recurrence_frequency_e freq);
static void _append_items(void);

/**
 * @brief Creates event edition view.
 * @param[in] title The view title to be displayed.
 * @param[in] navi Main naviframe object.
 * @param[in] rec_init The pointer to the record information structure.
 * The data contained in this structure is used to initialize the view components.
 * If this argument is set to NULL, then the view's widgets appear empty.
 * @param[in] func The callback function to be invoked on 'Save' button click.
 * @return This function returns a layout object if is was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_event_edit_create(const char *title, Evas_Object *navi, rec_info_s *rec_init, event_saved_cb func)
{
	int rec_id = 0;
	Evas_Object *button;
	Elm_Object_Item *item;
	time_t time_now = time(0);
	struct tm *loc_tm = localtime(&time_now);

	s_info.naviframe = navi;
	s_info.saved_cb = func;

	/* Set current time */
	if (loc_tm) {
		s_info.stm = *loc_tm;
		s_info.etm = *loc_tm;
	}

	s_info.layout = view_create_layout(s_info.naviframe, EDJ_EVENT_NEW, GRP_EVENT_NEW);
	if (!s_info.layout)
		return NULL;

	elm_layout_theme_set(s_info.layout, "layout", "application", "default");

	s_info.genlist = view_create_genlist(s_info.layout, PART_EVENT_NEW_LIST);
	if (!s_info.genlist)
		return NULL;

	_append_items();

	item = elm_naviframe_item_push(s_info.naviframe, title, NULL, NULL, s_info.layout, NULL);
	elm_naviframe_item_pop_cb_set(item, _navi_pop_cb, NULL);

	if (rec_init)
		rec_id = rec_init->id;

	button = view_create_button(s_info.naviframe, "Save", NULL, _save_button_clicked_cb, (void *)rec_id);
	if (!button)
		return NULL;

	elm_object_item_part_content_set(item, "toolbar", button);

	_view_initialize(rec_init);

	return s_info.layout;
}

/**
 * @brief Internal function witch validates correctness of event's title input.
 * @param[in] title The title string to validate.
 * @return This function returns copy of title string if it is valid,
 * otherwise NULL is returned.
 */
static char *_is_title_valid(const char *title)
{
	if (!title) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_is_title_valid() failed. Input argument is NULL.");
		return NULL;
	}

	if (strlen(title) == 0 || strlen(title) > 20)
		return NULL;

	while (isspace(*title))
		++title;

	if (*title != '\0')
		return strdup(title);

	return NULL;
}

/**
 * @brief Internal function witch validates correctness of event's start/end time.
 * This function checks whether t2 time is greater than t1 time.
 * @param[in] t1 The time which has to be smaller than t2.
 * @param[in] t2 The time which has to be greater than t1.
 * @return This function returns 'true' if the time t2 is greater than time t1,
 * otherwise 'false' is returned.
 */
static bool _is_datetime_valid(struct tm t1, struct tm t2)
{
	return (difftime(mktime(&t2), mktime(&t1)) > 0);
}

/**
 * @brief Internal callback function invoked on top item removal from the naviframe.
 * This function is responsible for active popup deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] it The naviframe item being popped.
 * @return This function returns 'EINA_TRUE' to allow naviframe item popping
 * or 'EINA_FALSE' to cancel this operation.
 */
static Eina_Bool _navi_pop_cb(void *data, Elm_Object_Item *it)
{
	free(s_info.title_text);
	free(s_info.location_text);
	free(s_info.description_text);

	s_info.title_text = NULL;
	s_info.location_text = NULL;
	s_info.description_text = NULL;

	_popup_cancel_click_cb(NULL, NULL, NULL);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on genlist scroll action.
 * This function is responsible for view scrolling, so the input field is visible.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _scroll_genlist_top_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (s_info.genlist_location_item == (Elm_Object_Item *)event_info ||
		s_info.genlist_description_item == (Elm_Object_Item *)event_info ||
		s_info.genlist_reminder_item == (Elm_Object_Item *)event_info)
		elm_genlist_item_show((Elm_Object_Item *)event_info, ELM_GENLIST_ITEM_SCROLLTO_MIDDLE);
}

/**
 * @brief Internal callback function invoked on entry text change.
 * This function is responsible for entry text storage.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	const char *text = elm_object_text_get(obj);
	char **storage = (char **)data;

	free(*storage);

	if (text)
		*storage = strdup(text);
}

/**
 * @brief Internal callback function invoked on 'Save' button click.
 * This function is responsible for event creation based on input data.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _save_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	int rec_id = (int)data;
	char *title;
	bool do_update;
	calendar_record_h rec_h;

	elm_toolbar_item_selected_set(event_info, EINA_FALSE);

	title = _is_title_valid(s_info.title_text);
	if (!title) {
		s_info.popup = view_create_popup("Warning", "Please enter the title up to 20 characters.", _popup_cancel_click_cb, NULL);
		return;
	}

	if (!_is_datetime_valid(s_info.stm, s_info.etm)) {
		s_info.popup = view_create_popup("Warning", "Invalid time period selected.", _popup_cancel_click_cb, NULL);
		free(title);
		return;
	}

	if (data_connect_calendar()) {
		do_update = (rec_id > 0 && data_get_record(rec_id, &rec_h));
		if (!do_update && !data_create_record(&rec_h)) {
			free(title);
			data_disconnect_calendar();
			return;
		}

		if (data_set_record_data(rec_h,
									title,
									&s_info.stm, &s_info.etm, s_info.is_allday,
									s_info.location_text, s_info.description_text,
									(int)s_info.priority, (int)s_info.sensitivity, (int)s_info.status,
									s_info.recurrence_data.id, s_info.recurrence_data.range_type, s_info.recurrence_data.count, &s_info.recurrence_data.tm,
									s_info.reminder)) {
			if (do_update)
				data_update_record(rec_h);
			else
				data_insert_record(rec_h);
		}

		data_destroy_record(rec_h);
		data_disconnect_calendar();
	}

	free(title);
	elm_naviframe_item_pop(s_info.naviframe);

	if (s_info.saved_cb)
		s_info.saved_cb();
}

/**
 * @brief Internal callback function invoked on 'Cancel' button click from popup window.
 * This function is responsible for active popup deletion.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_cancel_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (!s_info.popup)
		return;

	evas_object_del(s_info.popup);
	s_info.popup = NULL;
}

/**
 * @brief Internal callback function invoked on 'OK' button click from popup window.
 * This function is responsible for start/end date/time acquisition from setting controls
 * and data structure alteration.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_ok_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *selected_item = elm_genlist_selected_item_get(s_info.genlist);
	Evas_Object *datetime = elm_object_part_content_get(s_info.popup, "default");
	if (!datetime) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_object_part_content_get() failed.");
		return;
	}

	if (selected_item == s_info.genlist_from_item)
		elm_datetime_value_get(datetime, &s_info.stm);
	else if (selected_item == s_info.genlist_to_item)
		elm_datetime_value_get(datetime, &s_info.etm);

	elm_genlist_item_update(selected_item);
	_popup_cancel_click_cb(data, obj, event_info);
}

/**
 * @brief Internal callback function invoked on start/end date/time button click.
 * This function is responsible for popup creation with event's start/end date/time setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _datetime_button_click_cb(void *data, Evas_Object* obj, void *event_info)
{
	Evas_Object *datetime;
	const char *layout = (const char *)data;

	s_info.popup = view_create_popup(NULL, NULL, _popup_ok_click_cb, _popup_cancel_click_cb);
	if (!s_info.popup)
		return;

	datetime = view_create_datetime(s_info.popup, "default", layout, &s_info.recurrence_data.tm);
	if (!datetime) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Cannot create datetime widget!");
		evas_object_del(s_info.popup);
		s_info.popup = NULL;
		return;
	}
}

/**
 * @brief Internal callback function invoked on 'All day' check state change.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] it The naviframe item being popped.
 * @return This function returns 'EINA_TRUE' to allow naviframe item popping
 * or 'EINA_FALSE' to cancel this operation.
 */
static void _allday_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_info.is_allday = !s_info.is_allday;

	elm_genlist_item_update(s_info.genlist_from_item);
	elm_genlist_item_update(s_info.genlist_to_item);
}

/**
 * @brief Internal callback function invoked on reminder list item click.
 * This function is responsible for reminder alarm adjustment.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _reminder_list_item_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	label_type_s *label_s = (label_type_s *)data;

	elm_object_text_set(s_info.reminder_btn, label_s->name);
	evas_object_size_hint_min_set(s_info.reminder_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

	s_info.reminder = label_s->value;
	s_info.reminder_type_name = label_s->name;
	_popup_cancel_click_cb(NULL, NULL, NULL);
}

/**
 * @brief Internal callback function invoked on reminder button click.
 * This function is responsible for popup creation with event's reminder setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _reminder_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	int i;
	s_info.popup = view_create_ctxpopup(obj);
	if (!s_info.popup)
		return;

	for (i = 0; i < sizeof(s_info.reminder_labels) / sizeof(s_info.reminder_labels[0]); i++)
		elm_ctxpopup_item_append(s_info.popup, s_info.reminder_labels[i].name, NULL, _reminder_list_item_click_cb, (void *)&s_info.reminder_labels[i]);

	evas_object_show(s_info.popup);
}

/**
 * @brief Internal callback function invoked on recurrence setting change.
 * This function is responsible for recurrence button's caption update.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _recurrence_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_info.recurrence_type_name = _get_recurrence_label(s_info.recurrence_data.id);

	elm_object_text_set(s_info.recurrence_btn, s_info.recurrence_type_name);
	evas_object_size_hint_min_set(s_info.recurrence_btn, BUTTON_WIDTH, BUTTON_HEIGHT);
}

/**
 * @brief Internal callback function invoked on recurrence button click.
 * This function is responsible for recurrence view creation.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _recurrence_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	view_recurrence_create(s_info.naviframe, &s_info.recurrence_data, _recurrence_changed_cb);
}

/**
 * @brief Internal callback function invoked on priority button click.
 * This function is responsible for popup creation with event's priority setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _priority_list_item_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	label_type_s *label_s = (label_type_s *)data;

	elm_object_text_set(s_info.priority_btn, label_s->name);
	evas_object_size_hint_min_set(s_info.priority_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

	s_info.priority = label_s->value;
	s_info.priority_type_name = label_s->name;
	_popup_cancel_click_cb(NULL, NULL, NULL);
}

/**
 * @brief Internal callback function invoked on priority button click.
 * This function is responsible for popup creation with event's priority setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _priority_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	int i;
	s_info.popup = view_create_ctxpopup(obj);
	if (!s_info.popup)
		return;

	for (i = 0; i < sizeof(s_info.priority_labels) / sizeof(s_info.priority_labels[0]); i++)
		elm_ctxpopup_item_append(s_info.popup, s_info.priority_labels[i].name, NULL, _priority_list_item_click_cb, (void *)&s_info.priority_labels[i]);

	evas_object_show(s_info.popup);
}

/**
 * @brief Internal callback function invoked on sensitivity button click.
 * This function is responsible for popup creation with event's sensitivity setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _sensitivity_list_item_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	label_type_s *label_s = (label_type_s *)data;

	elm_object_text_set(s_info.sensitivity_btn, label_s->name);
	evas_object_size_hint_min_set(s_info.sensitivity_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

	s_info.sensitivity = label_s->value;
	s_info.sensitivity_type_name = label_s->name;
	_popup_cancel_click_cb(NULL, NULL, NULL);
}

/**
 * @brief Internal callback function invoked on sensitivity button click.
 * This function is responsible for popup creation with event's sensitivity setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _sensitivity_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	int i;
	s_info.popup = view_create_ctxpopup(obj);
	if (!s_info.popup)
		return;

	for (i = 0; i < sizeof(s_info.sensitivity_labels) / sizeof(s_info.sensitivity_labels[0]); i++)
		elm_ctxpopup_item_append(s_info.popup, s_info.sensitivity_labels[i].name, NULL, _sensitivity_list_item_click_cb, (void *)&s_info.sensitivity_labels[i]);

	evas_object_show(s_info.popup);
}

/**
 * @brief Internal callback function invoked on sensitivity button click.
 * This function is responsible for popup creation with event's sensitivity setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _status_list_item_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	label_type_s *label_s = (label_type_s *)data;

	elm_object_text_set(s_info.status_btn, label_s->name);
	evas_object_size_hint_min_set(s_info.status_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

	s_info.status = label_s->value;
	s_info.status_type_name = label_s->name;
	_popup_cancel_click_cb(NULL, NULL, NULL);
}

/**
 * @brief Internal callback function invoked on status button click.
 * This function is responsible for popup creation with event's status setting controls.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _status_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	int i;
	s_info.popup = view_create_ctxpopup(obj);
	if (!s_info.popup)
		return;

	for (i = 0; i < sizeof(s_info.status_labels) / sizeof(s_info.status_labels[0]); i++)
		elm_ctxpopup_item_append(s_info.popup, s_info.status_labels[i].name, NULL, _status_list_item_click_cb, (void *)&s_info.status_labels[i]);

	evas_object_show(s_info.popup);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * The text passed to the _append_item() function is displayed.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_get_text_cb(void *data, Evas_Object *obj, const char *part)
{
	if (!data)
		return NULL;

	return strdup((char *)data);
}

/**
 * @brief Internal callback function invoked on genlist item display (title).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains an entry widget for title setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_title_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	Evas_Object *layout = view_create_edit_field(obj, "Event's title");
	s_info.entry_title = elm_object_part_content_get(layout, "elm.swallow.content");

	if (s_info.title_text)
		elm_object_text_set(s_info.entry_title, s_info.title_text);

	evas_object_smart_callback_add(s_info.entry_title, "changed", _entry_changed_cb, (void *)&s_info.title_text);

	return layout;
}

/**
 * @brief Internal callback function invoked on genlist item display (start time).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widgets for start date/time setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_start_datetime_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	static date_time_s date_setting = {.tm = &s_info.stm, .format = "%d %b %Y" };
	static date_time_s time_setting = {.tm = &s_info.stm, .format = "%R" };
	Evas_Object *box;
	char time_str[64] = {0};
	Evas_Object *button;

	if (!strcmp(part, "elm.swallow.end")) {
		box = view_create_box(obj);
		if (!box)
			return NULL;

		strftime(time_str, sizeof(time_str), date_setting.format, &s_info.stm);
		button = view_create_button(box, time_str, NULL, _datetime_button_click_cb, (void *)"date_layout");
		if (!button)
			return NULL;

		evas_object_size_hint_min_set(button, BUTTON_DATE_WIDTH, BUTTON_HEIGHT);
		elm_box_pack_end(box, button);

		if (!s_info.is_allday) {
			strftime(time_str, sizeof(time_str), time_setting.format, &s_info.stm);
			button = view_create_button(box, time_str, NULL, _datetime_button_click_cb, (void *)"time_layout");
			if (!button)
				return NULL;

			evas_object_size_hint_min_set(button, BUTTON_TIME_WIDTH, BUTTON_HEIGHT);
			elm_box_pack_end(box, button);
		}

		return box;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (end time).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widgets for end date/time setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_end_datetime_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	static date_time_s date_setting = {.tm = &s_info.etm, .format = "%d %b %Y" };
	static date_time_s time_setting = {.tm = &s_info.etm, .format = "%R" };
	Evas_Object *box;
	char time_str[64] = {0};
	Evas_Object *button;

	if (!strcmp(part, "elm.swallow.end")) {
		box = view_create_box(obj);
		if (!box)
			return NULL;

		strftime(time_str, sizeof(time_str), date_setting.format, &s_info.etm);
		button = view_create_button(box, time_str, NULL, _datetime_button_click_cb, (void *)"date_layout");
		if (!button)
			return NULL;

		evas_object_size_hint_min_set(button, BUTTON_DATE_WIDTH, BUTTON_HEIGHT);
		elm_box_pack_end(box, button);

		if (!s_info.is_allday) {
			strftime(time_str, sizeof(time_str), time_setting.format, &s_info.etm);
			button = view_create_button(box, time_str, NULL, _datetime_button_click_cb, (void *)"time_layout");
			if (!button)
				return NULL;

			evas_object_size_hint_min_set(button, BUTTON_TIME_WIDTH, BUTTON_HEIGHT);
			elm_box_pack_end(box, button);
		}

		return box;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (timezone).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget for end time setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_timezone_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	char buff[64] = {'\0'};
	time_t local_time = time(NULL);
	struct tm *tm_localtime = localtime(&local_time);
	Evas_Object *label;

	if (!strcmp(part, "elm.swallow.end")) {
		strftime(buff, sizeof(buff), "%Z %z", tm_localtime);
		label = view_create_label(obj, buff, NULL);
		if (!label)
			return NULL;

		return label;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (all day flag).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a check box widget for all day flag setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_allday_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	Evas_Object *check = NULL;

	if (!strcmp(part, "elm.swallow.end")) {
		s_info.is_allday = EINA_FALSE;
		check = view_create_check(obj, s_info.is_allday, _allday_changed_cb);
	}

	return check;
}

/**
 * @brief Internal callback function invoked on genlist item display (location).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains an entry widget for location setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_location_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	Evas_Object *layout = view_create_edit_field(obj, "Event's location");
	s_info.entry_location = elm_object_part_content_get(layout, "elm.swallow.content");

	if (s_info.location_text)
		elm_object_text_set(s_info.entry_location, s_info.location_text);

	evas_object_smart_callback_add(s_info.entry_location, "changed", _entry_changed_cb, (void *)&s_info.location_text);

	return layout;
}

/**
 * @brief Internal callback function invoked on genlist item display (description).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains an entry widget for description setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_description_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	Evas_Object *layout = view_create_edit_field(obj, "Event's description");
	s_info.entry_description = elm_object_part_content_get(layout, "elm.swallow.content");

	if (s_info.description_text)
		elm_object_text_set(s_info.entry_description, s_info.description_text);

	evas_object_smart_callback_add(s_info.entry_description, "changed", _entry_changed_cb, (void *)&s_info.description_text);

	return layout;
}

/**
 * @brief Internal callback function invoked on genlist item display (reminder).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget for reminder setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_reminder_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	if (!strcmp(part, "elm.swallow.end")) {
		s_info.reminder_btn = view_create_button(obj, s_info.reminder_type_name, NULL, _reminder_button_click_cb, NULL);
		if (!s_info.reminder_btn)
			return NULL;

		evas_object_size_hint_min_set(s_info.reminder_btn, BUTTON_WIDTH, BUTTON_HEIGHT);
		return s_info.reminder_btn;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (recurrence).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget for recurrence setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_recurrence_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	if (!strcmp(part, "elm.swallow.end")) {
		s_info.recurrence_btn = view_create_button(obj, s_info.recurrence_type_name, NULL, _recurrence_button_click_cb, NULL);
		if (!s_info.recurrence_btn)
			return NULL;

		evas_object_size_hint_min_set(s_info.recurrence_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

		return s_info.recurrence_btn;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (priority).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget for priority setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_priority_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	if (!strcmp(part, "elm.swallow.end")) {
		s_info.priority_btn = view_create_button(obj, s_info.priority_type_name, NULL, _priority_button_click_cb, NULL);
		if (!s_info.priority_btn)
			return NULL;

		evas_object_size_hint_min_set(s_info.priority_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

		return s_info.priority_btn;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (sensitivity).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget for sensitivity setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_sensivity_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	if (strcmp(part, "elm.swallow.end") == 0) {
		s_info.sensitivity_btn = view_create_button(obj, s_info.sensitivity_type_name, NULL, _sensitivity_button_click_cb, NULL);
		if (!s_info.sensitivity_btn)
			return NULL;

		evas_object_size_hint_min_set(s_info.sensitivity_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

		return s_info.sensitivity_btn;
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display (status).
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget for status setting.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_get_status_content_cb(void *item_data, Evas_Object *obj, const char *part)
{
	if (strcmp(part, "elm.swallow.end") == 0) {
		s_info.status_btn = view_create_button(obj, s_info.status_type_name, NULL, _status_button_click_cb, NULL);
		if (!s_info.status_btn)
			return NULL;

		evas_object_size_hint_min_set(s_info.status_btn, BUTTON_WIDTH, BUTTON_HEIGHT);

		return s_info.status_btn;
	}

	return NULL;
}

/**
 * @brief Internal function which appends new item to the genlist.
 * @param[in] item_style The name of the style to be applied to the item.
 * @param[in] item_text The text to be displayed on the item text label.
 * @param[in] on_text_get_cb The callback function invoked on item's text redraw.
 * This callback function is assigned to the func.text_get member of the genlist item class structure.
 * @param[in] on_content_get_cb The callback function invoked on item's content redraw.
 * This callback function is assigned to the func.content_get member of the genlist item class structure.
  * @return The function returns the item object if it was successfully appended to the genlist,
 * otherwise NULL is returned.
 */
static Elm_Object_Item *_append_item(const char *item_style, const char *item_text, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb, Elm_Genlist_Item_Content_Get_Cb on_content_get_cb)
{
	Elm_Genlist_Item_Class *itc;
	Elm_Object_Item *item;

	itc = create_genlist_item_class(item_style, on_text_get_cb, on_content_get_cb);
	if (!itc)
		return NULL;

	item = elm_genlist_item_append(s_info.genlist, itc, (void *)item_text, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_append() failed.");
		return NULL;
	}

	elm_genlist_item_class_free(itc);

	return item;
}

/**
 * @brief Internal function which gets the text label from provided array of labels
 * with specified identifier.
 * @param[in] labels The array of labels.
 * @param[in] labels_count The number of items in the array.
 * @param[in] value The value for label identification.
 * @return This function returns the label's text identified by value.
 * If the label for specified identifier does not exist, then empty string is returned.
 */
static const char *_get_feature_label(label_type_s *labels, int labels_count, int value)
{
	int i;

	for (i = 0; i < labels_count; i++)
		if (labels[i].value == value)
			return labels[i].name;

	return "";
}

/**
 * @brief Internal function which sets the initial values to the view's components.
 * @param[in] rec_info The record information structure which is used as a basis for
 * widgets content fill up.
 */
static void _view_initialize(rec_info_s *rec_info)
{
	if (!rec_info)
		return;

	s_info.stm = rec_info->t_start;
	s_info.etm = rec_info->t_end;
	s_info.is_allday = rec_info->all_day;
	s_info.recurrence_type_name = _get_recurrence_label(rec_info->recurr_freq);
	s_info.recurrence_data.id = rec_info->recurr_freq;
	s_info.recurrence_data.range_type = rec_info->recurr_range;
	s_info.recurrence_data.count = rec_info->recurr_count;
	s_info.recurrence_data.tm = rec_info->recurr_until;
	s_info.reminder = rec_info->reminder;
	s_info.priority = rec_info->priority;
	s_info.sensitivity = rec_info->sensitivity;
	s_info.status = rec_info->status;
	s_info.reminder_type_name = _get_feature_label(s_info.reminder_labels, sizeof(s_info.reminder_labels), rec_info->reminder);
	s_info.priority_type_name = _get_feature_label(s_info.priority_labels, sizeof(s_info.priority_labels), rec_info->priority);
	s_info.sensitivity_type_name = _get_feature_label(s_info.sensitivity_labels, sizeof(s_info.sensitivity_labels), rec_info->sensitivity);
	s_info.status_type_name = _get_feature_label(s_info.status_labels, sizeof(s_info.status_labels), rec_info->status);

	if (rec_info->title)
		s_info.title_text = strdup(rec_info->title);

	if (rec_info->location)
		s_info.location_text = strdup(rec_info->location);

	if (rec_info->description)
		s_info.description_text = strdup(rec_info->description);
}

/**
 * @brief Internal function which gets the name of the recurrence type.
 * @param[in] freq The recurrence frequency.
 * @return This function returns the name of the provided recurrence scheme.
 */
static const char *_get_recurrence_label(calendar_recurrence_frequency_e freq)
{
	switch (freq) {
	case CALENDAR_RECURRENCE_NONE:
		return "None";
	case CALENDAR_RECURRENCE_DAILY:
		return "Daily";
	case CALENDAR_RECURRENCE_WEEKLY:
		return "Weekly";
	case CALENDAR_RECURRENCE_MONTHLY:
		return "Monthly";
	case CALENDAR_RECURRENCE_YEARLY:
		return "Yearly";
	default:
		return "Undefined";
	}
}

/**
 * @brief Internal function which appends the set of items to the genlist.
 * @return This function returns 'true' if all the items were successfully appended to the genlist,
 * otherwise 'false' is returned.
 */
static void _append_items(void)
{
	_append_item("full", NULL, NULL, _get_title_content_cb);

	s_info.genlist_from_item = _append_item("default", "Start", _get_text_cb, _get_start_datetime_content_cb);
	s_info.genlist_to_item = _append_item("default", "End", _get_text_cb, _get_end_datetime_content_cb);

	_append_item("default", "Time zone", _get_text_cb, _get_timezone_content_cb);
	_append_item("default", "All day", _get_text_cb, _get_allday_content_cb);

	s_info.genlist_location_item = _append_item("full", NULL, NULL, _get_location_content_cb);
	s_info.genlist_description_item = _append_item("full", NULL, NULL, _get_description_content_cb);
	s_info.genlist_reminder_item = _append_item("default", "Reminder", _get_text_cb, _get_reminder_content_cb);

	_append_item("default", "Recurrence", _get_text_cb, _get_recurrence_content_cb);
	_append_item("default", "Priority", _get_text_cb, _get_priority_content_cb);
	_append_item("default", "Sensitivity", _get_text_cb, _get_sensivity_content_cb);
	_append_item("default", "Status", _get_text_cb, _get_status_content_cb);

	evas_object_smart_callback_add(s_info.genlist, "selected", _scroll_genlist_top_cb, NULL);
}
