/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <time.h>
#include <efl_extension.h>
#include "$(appName).h"
#include "view.h"
#include "view_defines.h"
#include "view/view_main.h"
#include "view/view_event_edit.h"
#include "view/view_event_details.h"
#include "data.h"

#define FONT_SIZE 40

static struct view_info {
	Evas_Object *win;
	Evas_Object *navi;
	Evas_Object *all_day_list;
	Evas_Object *events_list;
	Elm_Object_Item *navi_item;
	struct tm curr_time;
} s_info = {
	.win = NULL,
	.navi = NULL,
	.all_day_list = NULL,
	.events_list = NULL,
	.navi_item = NULL,
	.curr_time = {0,},
};

static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info);
static void _prev_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _next_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _create_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _new_event_saved_cb(void);
static void _event_deleted_cd(void);
static Evas_Object *_create_main_layout(Evas_Object *parent);
static void _get_events(Evas_Object *gen_list, bool all_day);
static void _refresh_events_list(void);
static void _set_page_title(void);

/**
 * @brief Creates the main application's layout.
 * @param[in] navi The application's naviframe.
 * @param[in] navi_item The top item of naviframe.
 * @return This function returns the main layout object if it is created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_main_create(Evas_Object *navi, Elm_Object_Item *navi_item)
{
	time_t t;
	Evas_Object *button;
	Evas_Object *layout;

	time(&t);

	s_info.navi = navi;
	s_info.navi_item = navi_item;
	s_info.curr_time = *localtime(&t);

	layout = _create_main_layout(s_info.navi);
	if (!layout)
		return NULL;

	elm_object_item_part_content_set(s_info.navi_item, "default", layout);

	button = view_create_button(s_info.navi, "Create", NULL, _create_button_click_cb, NULL);
	if (!button)
		return NULL;

	elm_object_item_part_content_set(s_info.navi_item, "toolbar", button);

	_set_page_title();
	_refresh_events_list();

	return layout;
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	_refresh_events_list();
	elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal callback function invoked on 'Previous' button click.
 * Internally, this function obtains the previous events against currently displayed events.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _prev_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_info.curr_time.tm_mday -= 1;
	mktime(&s_info.curr_time);

	_set_page_title();
	_refresh_events_list();
}

/**
 * @brief Internal callback function invoked on 'Next' button click.
 * Internally, this function obtains the next events against currently displayed events.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _next_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_info.curr_time.tm_mday += 1;
	mktime(&s_info.curr_time);

	_set_page_title();
	_refresh_events_list();
}

/**
 * @brief Internal callback function invoked on 'Create' button click.
 * Internally, this function switches to the event creation view.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _create_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *layout = view_event_edit_create("Add event", s_info.navi, NULL, _new_event_saved_cb);
	eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);
}

/**
 * @brief Internal callback function invoked by the new event creation view module
 * on successful event creation.
 */
static void _new_event_saved_cb(void)
{
	_refresh_events_list();
}

/**
 * @brief Internal callback function invoked by the event details view module
 * on successful event removal.
 */
static void _event_deleted_cd(void)
{
	_refresh_events_list();
}

/**
 * @brief Internal callback function invoked on events list item selection.
 * This function displays the selected event's details.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *layout;
	Elm_Object_Item *item = elm_genlist_selected_item_get(obj);
	rec_info_s *rec_info = (rec_info_s *)data;

	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (!rec_info) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_item_selected_cb() failed. Invalid input argument.");
		return;
	}

	layout = view_event_details_create(s_info.navi, rec_info, _event_deleted_cd);
	eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_get_text_cb(void *data, Evas_Object *obj, const char *part)
{
	char t_start_str[16] = {0,};
	char t_end_str[16] = {0,};
	char buff[36] = {0,};
	rec_info_s *rec_info = (rec_info_s *)data;

	if (!rec_info) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_get_text_cb() failed. Invalid input argument.");
		return NULL;
	}

	if (!strcmp(part, "elm.text")) {
		return strdup(rec_info->title);
	} else if (!strcmp(part, "elm.text.multiline")) {
		strftime(t_start_str, sizeof(t_start_str), " %d/%m/%Y", &rec_info->t_start);
		strftime(t_end_str, sizeof(t_end_str), " %d/%m/%Y", &rec_info->t_end);
		snprintf(buff, sizeof(buff), "%s - %s", t_start_str, t_end_str);
		return strdup(buff);
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item removal.
 * This function is assigned to the func.del handler of genlist item's class structure.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static void _item_del_cb(void *data, Evas_Object *obj)
{
	rec_info_s *rec_info = (rec_info_s *)data;

	if (!rec_info) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_item_del_cb() failed. Invalid input argument.");
		return;
	}

	data_destroy_record_data(rec_info);
}

/**
 * @brief Internal function which creates the main application's layout set to the item of naviframe object.
 * @param[in] parent The parent object for the layout.
 * @return This function returns a layout object if is was created successfully,
 * otherwise NULL is returned.
 */
static Evas_Object *_create_main_layout(Evas_Object *parent)
{
	Evas_Object *layout;
	Evas_Object *toolbar;

	layout = view_create_layout(parent, EDJ_MAIN, GRP_MAIN);
	if (!layout)
		return NULL;

	if (!view_create_button(layout, "Previous", PART_MAIN_BUTTON_PREV, _prev_button_click_cb, NULL))
		return NULL;

	if (!view_create_button(layout, "Next", PART_MAIN_BUTTON_NEXT, _next_button_click_cb, NULL))
		return NULL;

	if (!view_create_label(layout, "All Day Events", PART_MAIN_LABEL_ALL_DAY))
		return NULL;

	if (!view_create_label(layout, "Events", PART_MAIN_LABEL_EVENTS))
		return NULL;

	s_info.all_day_list = view_create_genlist(layout, PART_MAIN_LIST_ALL_DAY);
	if (!s_info.all_day_list)
		return NULL;

	s_info.events_list = view_create_genlist(layout, PART_MAIN_LIST_EVENTS);
	if (!s_info.events_list)
		return NULL;

	toolbar = view_create_toolbar(layout, PART_MAIN_TOOLBAR);
	if (!toolbar)
		return NULL;

	return layout;
}

/**
 * @brief Internal function which gets all the records stored in calendar database
 * and displays them on the provided genlist widget.
 * @param[in] gen_list The target genlist to display the calendar events.
 * @param[in] all_day The flag for non/all-day events filtering.
 */
static void _get_events(Evas_Object *gen_list, bool all_day)
{
	int count = 0;
	double t_diff;
	rec_info_s *rec_info;
	calendar_list_h list_h;
	calendar_record_h rec_h;
	Elm_Genlist_Item_Class *itc;

	elm_genlist_clear(gen_list);

	if (!data_connect_calendar())
		return;

	if (!data_get_all_records(_calendar_event._uri, &list_h, &count) ||
		!data_get_first_list_item(list_h, &rec_h))
		return;

	if (count == 0 || !rec_h) {
		dlog_print(DLOG_WARN, LOG_TAG, "No records in database.");
		calendar_list_destroy(list_h, true);
		data_disconnect_calendar();
		return;
	}

	itc = create_genlist_item_class("multiline", _get_text_cb, NULL);
	if (!itc) {
		calendar_list_destroy(list_h, true);
		data_disconnect_calendar();
		return;
	}

	itc->func.del = _item_del_cb;

	do {
		rec_info = NULL;
		if (!data_get_record_data(rec_h, &rec_info))
			continue;

		t_diff = difftime(mktime(&s_info.curr_time), mktime(&rec_info->t_start));

		if (t_diff > 0 || all_day != rec_info->all_day) {
			data_destroy_record_data(rec_info);
			continue;
		}

		elm_genlist_item_append(gen_list, itc, (void *)rec_info, NULL, ELM_GENLIST_ITEM_NONE, _item_selected_cb, (void *)rec_info);
	} while (data_get_next_list_item(list_h, &rec_h));

	calendar_list_destroy(list_h, true);
	data_disconnect_calendar();
	elm_genlist_item_class_free(itc);
}

/**
 * @brief Internal function which clears the events lists and populates them again.
 */
static void _refresh_events_list(void)
{
	_get_events(s_info.all_day_list, true);
	_get_events(s_info.events_list, false);
}

/**
 * @brief Internal function which displays the date in the page title.
 */
static void _set_page_title(void)
{
	char time_str[64] = {0,};

	strftime(time_str, sizeof(time_str), "%d %b %Y", &s_info.curr_time);
	elm_object_item_part_text_set(s_info.navi_item, "default", time_str);
}
